#!/bin/sh

LS=/usr/bin/ls
RM=/usr/bin/rm
AWK=/usr/bin/awk
CAT=/usr/bin/cat
SED=/usr/bin/sed
MDB="/usr/bin/mdb -k"
ZFS=/sbin/zfs
GREP=/usr/bin/grep
KSTAT=/usr/bin/kstat
ZPOOL=/sbin/zpool
IOSTAT=/usr/bin/iostat
FMTOPO=/usr/lib/fm/fmd/fmtopo
SMARTCTL=/usr/sbin/smartctl

F_FMTOPO=/tmp/fmtopo.$$
F_IOSTAT=/tmp/iostat.$$
F_SMART=/tmp/smart.$$

parse_fmtopo()
{

	${FMTOPO} 'hc://*/disk=0'|${AWK} -F':' '{print $6,$8}'|while read s r; do
		[ -z "${s}" ] && continue
		serial=`echo "$s"|${AWK} -F'=' '{print $2}'`
		enc=`echo "$r"|${AWK} -F'/' '{print $2}'|${AWK} -F'=' '{print $2}'`
		slot=`echo "$r"|${AWK} -F'/' '{print $3}'|${AWK} -F'=' '{print $2}'`
		echo ${serial} ${enc} ${slot}
	done
}


read_smart()
{
	local vdev=`echo ${d}|${SED} -e 's,.*/c,c,g;s,s0$,,g'`
	local file=/tmp/disk_${vdev}.$$

	${SMARTCTL} -a ${1} >${file}
}


# Разбор файла вывода smartctl на поля
#
# $1	- имя файла
# $2	- имя поля (разделитель :)
parse_smart1()
{

	${GREP} -Pi "${2}" ${1}|${AWK} -F':' '{gsub(/^[ \t]+/,"",$2);gsub(/[ \t]+$/,"",$2);print $2}'
}


# Разбор файла вывода smartctl на поля
#
# $1	- имя файла
# $2	- имя поля
# $3	- номер интересующего поля
parse_smart2()
{
	local num="\$${3}"

	${GREP} -Pi "${2}" ${1}|${AWK} "{print $num}"
}


parse_scan()
{
	local scan=/tmp/scan.$$
	local serial

	${SMARTCTL} --scan-open|${AWK} '{print $1}'>${scan}
	while read d; do
		read_smart ${d}&
	done <${scan}
	${RM} -f ${scan}
	wait
	${LS} -1 /tmp/disk_*.$$|while read f; do
		serial=`parse_smart1 ${f} "^Serial number"`
		[ -z "${serial}" ] && continue
		vdev=`echo ${f}|${SED} -e 's,.*/disk_,,g;s,\..*,,g'`
		echo "disk/${vdev}/serial:${serial}"
		str=`${GREP} ${serial} ${F_FMTOPO}`
		if [ ! -z "${str}" ]; then
			enc=`echo ${str}|${AWK} '{print $2}'`
			echo "disk/${vdev}/enclosure:${enc}"
			slot=`echo ${str}|${AWK} '{print $3}'`
			echo "disk/${vdev}/slot:${slot}"
		fi
		sd=`${IOSTAT} -E ${vdev}|${AWK} '/^sd/{print $1}'`
		[ ! -z "${sd}" ] && echo "disk/${vdev}/sd:${sd}"
		vid=`parse_smart1 ${f} "^Vendor"`
		[ -z "${vid}" ] && vid=`parse_smart1 ${f} "^Model Family"`
		[ ! -z "${vid}" ] && echo "disk/${vdev}/vid:${vid}"
		pid=`parse_smart1 ${f} "^Product"`
		[ -z "${pid}" ] && pid=`parse_smart1 ${f} "^Device Model"`
		[ ! -z "${pid}" ] && echo "disk/${vdev}/pid:${pid}"
		rev=`parse_smart1 ${f} "^Revision"`
		[ ! -z "${rev}" ] && echo "disk/${vdev}/revision:${rev}"
		fw=`parse_smart1 ${f} "^Firmware Version"`
		[ ! -z "${fw}" ] && echo "disk/${vdev}/firmware:${fw}"
		capacity=`parse_smart1 ${f} "^User Capacity"|${SED} -e 's/bytes.*//g;s/ //g;s/,//g'`
		echo "disk/${vdev}/capacity:${capacity}"
		lsize=`parse_smart1 ${f} "^Logical block size"`
		psize=`parse_smart1 ${f} "^Physical block size"`
		if [ -z "${lsize}" ] && [ -z "${psize}" ]; then
			secsize=`parse_smart1 ${f} "^Sector Sizes"`
			lsize=`echo ${secsize}|${SED} -e 's/bytes.*//g'`
			psize=`echo ${secsize}|${SED} -e 's/.*, //g;s/bytes.*//g'`
		else
			lsize=`echo ${lsize}|${SED} -e 's/ .*//g'`
			if [ -z "${psize}" ]; then
				psize=${lsize}
			else
				psize=`echo ${psize}|${SED} -e 's/ .*//g'`
			fi
		fi
		echo "disk/${vdev}/logical:${lsize}"
		echo "disk/${vdev}/physical:${psize}"

		rrate=`parse_smart1 ${f} "^Rotation Rate"`
		if [ "${rrate}" = "Solid State Device" ]; then
			echo "disk/${vdev}/rotation_rate:SSD"
			reloc=`parse_smart2 ${f} "Reallocated_Sector_Ct" 10`
			[ ! -z "${reloc}" ] && echo "disk/${vdev}/relocated_sectors:${reloc}"
			pfail=`parse_smart2 ${f} "Program_Fail_Count" 10`
			[ ! -z "${pfail}" ] && echo "disk/${vdev}/prog_fail:${pfail}"
			efail=`parse_smart2 ${f} "Erase_Fail_Count" 10`
			[ ! -z "${efail}" ] && echo "disk/${vdev}/erase_fail:${efail}"
			ete=`parse_smart2 ${f} "End-to-End_Error" 10`
			[ ! -z "${ete}" ] && echo "disk/${vdev}/ete_err:${ete}"
			uncor=`parse_smart2 ${f} "Reported_Uncorrect" 10`
			[ ! -z "${uncor}" ] && echo "disk/${vdev}/uncorrect:${uncor}"
			crc=`parse_smart2 ${f} "CRC_Error_Count" 10`
			[ ! -z "${crc}" ] && echo "disk/${vdev}/crc_error:${crc}"
		else
			rrate=`echo ${rrate}|${SED} -e 's/ rpm//g'`
			echo "disk/${vdev}/rotation_rate:${rrate}"
			egdl=`parse_smart1 ${f} "^Elements in grown defect list"`
			[ ! -z "${egdl}" ] && echo "disk/${vdev}/egdl:${egdl}"
			rd=`parse_smart2 ${f} "^read:" 8`
			[ ! -z "${rd}" ] && echo "disk/${vdev}/read:${rd}"
			wr=`parse_smart2 ${f} "^write:" 8`
			[ ! -z "${wr}" ] && echo "disk/${vdev}/write:${wr}"
			vf=`parse_smart2 ${f} "^verify:" 8`
			[ ! -z "${vf}" ] && echo "disk/${vdev}/verify:${vf}"
		fi
	done
	${RM} -f /tmp/disk_*.$$
}


parse_spa()
{
	local spa_addr
	local spa

	spa_addr=`echo '::walk spa'|${MDB}|${SED} -e 's/^0x//g'`
	for addr in ${spa_addr}; do
		susp=`echo "${addr}::print spa_t spa_suspended"|${MDB}|${AWK} '{print $3}'`
		[ ${susp} -ne 0 ] && return
	done
	${ZPOOL} listvdev|while read pool vdev status dummy; do
		if [ "X${spa}" != "X${pool}" ]; then
			spa=${pool}
			stp=`${ZPOOL} get -Hpo value health ${pool}`
			echo "pool/${pool}/status:${stp}"
			${ZFS} list -Hpo used,avail ${pool}|while read used avail; do
				echo "pool/${pool}/used:${used}"
				echo "pool/${pool}/avail:${avail}"
			done
		fi
		echo "pool/${pool}/vdev/${vdev}:${status}"
	done
}

echo "__CACHE_BEGIN__"
# uptime
${KSTAT} -Cp unix:0:system_misc:snaptime|${SED} -e 's/.*:/uptime\//g;s/,.*//g'

F_SPA=/tmp/zpool.$$

#zpool
parse_spa >${F_SPA}&
#fmtopo
parse_fmtopo >${F_FMTOPO}&
#smart
parse_scan
${CAT} ${F_SPA}
echo "__CACHE_END__"

${RM} -f ${F_FMTOPO} ${F_IOSTAT} ${F_SMART} ${F_SPA}
